function [imdec,cput]=deconv2d_term(x,h,deg,lambda,sigma)
% Term-by-term image deconvolution in the orthogonal Meyer wavelet domain (WaveD).
%
% 	Input
%		x	Input image.
%		h	PSF image.
%		deg	Degree of Meyer polynomial window 2 <= deg <= 4 (in fact, indexed fro 1 to 3).
%		lambda	Threshold parameter (default, 4.5).
%		sigma	Noise std (default, estimated by MAD).
%
%	Output
%		imdec	deconvolved image.
%		cput	A vector of CPU time of each step: 
%			+ cpu(1) Meyer wavelet transform.
%			+ cpu(2) thresholds computation.
%			+ cpu(3) denoise blocks.
%			+ cpu(4) inverse Meyer wavelet transform.
%

if (exist('x')~=1), 	error('Provide input image');	end;
if (exist('h')~=1), 	error('Provide input PSF');	end;
if (exist('deg')~=1), 	deg = 1; 			end;
if (exist('lambda')~=1),lambda = 4.5;			end;

[nx ny] = size(x);
if nx~=ny
	disp('Matrix must be square');
	return;
end

n 	= nx;
J	= nextpow2(n);
scale 	= max(3,floor(log2(floor(sqrt(2*J)))));   % Coarsest scale L%2^Jc matches the size of the block.

t = cputime;
X = fft2(x);
H = fft2(h);
Fh= X./H;
wc=FTWT2_YM(real(ifft2(Fh)),scale,deg);
cput(1) = cputime - t;

% Assuming the noise is iid Gaussian process and estimate the standard deviation as: sd % MAD(x)/0.6745
if (exist('sigma')~=1),
 ntx	= n/2;
 nty	= n/2;
 qmf	= MakeONFilter('Symmlet',6);
 buf	= FWT2_PO(x,J-1,qmf);
 hg	= reshape(buf(ntx+1:n,nty+1:n),ntx*nty,1);
 sigma	= MAD(hg);
 clear hg;
end

t = cputime;
thd= compute_thdMW(scale,J,deg,lambda,H);
cput(2) = cputime - t;
for j = 0:J-1-scale
 HH = wc(2^(J-j-1)+1 : 2^(J-j), 2^(J-j-1)+1 : 2^(J-j));
 HH = HardThresh(HH,thd{j+1}(3)*sigma);
 wc(2^(J-j-1)+1 : 2^(J-j), 2^(J-j-1)+1 : 2^(J-j)) = HH;
 
 HL = wc(2^(J-j-1)+1 : 2^(J-j), 1 : 2^(J-j-1));
 HL = HardThresh(HL,thd{j+1}(1)*sigma);
 wc(2^(J-j-1)+1 : 2^(J-j), 1 : 2^(J-j-1)) = HL;
 
 LH = wc(1 : 2^(J-j-1), 2^(J-j-1)+1 : 2^(J-j));
 LH = HardThresh(LH,thd{j+1}(2)*sigma);
 wc(1 : 2^(J-j-1), 2^(J-j-1)+1 : 2^(J-j)) = LH;
end
cput(3) = cputime - t;

t = cputime;
imdec = ITWT2_YM(wc,scale,deg);
cput(4) = cputime - t;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function thd = compute_thdMW(jc,J,deg,lambda,H)

phij = scaling_YM(jc,J,deg)';
wavfun = phij*phij';
thd{1} = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);

for j=jc:J-2
  phij = scaling_YM(j,J,deg)';
  psij = wavelet_YM(j,J,deg)';
  wavfun = psij*phij'; % HL.
  thd{j-jc+2}(1) = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);
  wavfun = phij*psij'; % LH.
  thd{j-jc+2}(2) = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);
  wavfun = psij*psij'; % HH.
  thd{j-jc+2}(3) = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);
end
phij = scaling_YM(J-1,J,deg)';
psij = fine_YM(J-1,J,deg)';
wavfun = psij*phij'; % HL.
thd{J-jc+1}(1) = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);
wavfun = phij*psij'; % LH.
thd{J-jc+1}(2) = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);
wavfun = psij*psij'; % HH.
thd{J-jc+1}(3) = lambda*norm(wavfun(wavfun~=0)./H(wavfun~=0))*2^(-J);
thd = fliplr(thd);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function phi = phiHAT(x,deg)
% auxillary function for computing Meyer scaling function
% in the Fourier domain

winh = WindowMeyer((3*abs(x)-1),deg);

cos_part = cos(pi*winh/2);

phi = zeros(1,length(x));
phi = cos_part.*(abs(x)>1/3 & abs(x)<=2/3);
phi(abs(x)<=1/3) = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = psiHAT(x,deg)
% auxillary function for computing Meyer wavelet function
% in the Fourier domain

wing = WindowMeyer((3*abs(x)-1),deg);
winh = WindowMeyer((3*abs(x)/2-1),deg);

gpart = (exp(-i*pi*x)).*sin(pi*wing/2);
hpart = (exp(-i*pi*x)).*cos(pi*winh/2);

y1 = gpart.*(abs(x)>1/3 & abs(x)<=2/3);
y2 = hpart.*(abs(x)>2/3 & abs(x)<=4/3);

psij = y1+y2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%3
function phij = scaling_YM(j,j_max,deg)
% Generate phiHAT(j,0)--the fft of the scaling function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;
aux1=phiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

phij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(-j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = wavelet_YM(j,j_max,deg)
% generate psiHAT(j,0)--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;

omega_neg = [-10/(2*pi):1/(2^j):-1/2^j];
n_omega = 2*length(omega_pos)-1;

n = 2^j_max-n_omega;

aux1 = psiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

psij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(-j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = fine_YM(j,j_max,deg)
% generate psiHAT--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).
% at the finest resolution level

omega = linspace(-pi,pi,2^(j_max));
phij = scaling_YM(j,j_max,deg);
psij = exp(-i*linspace(-pi,pi,2^(j_max))/2^j).*conj(fftshift(phij));

% omega_pos = [0:1/(2^j):10/(2*pi)];
% n_pos = 2^j-2^(j-2);
% 
% nn = 2^j_max;
% 
% aux1 = psiHAT(omega_pos,deg);
% 
% %Take zero out
% n2 = n_pos+2^(j-2)+1;
% psiHAT_LEFT=zeros(1,n2);
% 
% psiHAT_LEFT(1:n_pos) = aux1(1:n_pos);
% psiHAT_LEFT((n_pos+1):(n_pos+2^(j-2)+1)) = -1;
% aux2 = psiHAT_LEFT(2:end);
% 
% psiHAT_RIGHT = conj(fliplr(psiHAT_LEFT));
% psiHAT_RIGHT = psiHAT_RIGHT(2:end-1);
% 
% psij = [psiHAT_LEFT psiHAT_RIGHT]*2^(-j/2)*nn;

 

%
% Part of DBlockToolbox Version:100
% Written by: Jalal Fadili, GREYC CNRS ENSICAEN
%             Christophe Chesneau, LMNO University of Caen
%             Jean-Luc Starck, CEA-CNRS
% Created November 2008
% This material is distributed under CeCiLL licence.
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
% 
