function x_ward = forward(y,h,jc,qmfR1,qmfR2,regpar,thresh,sigma,psd)
% This program runs the ForWaRD algorithm to perform the deconvolution
% 
% Input
%	y	the Observation y.
%	h	the PSF.
%	jc	coarsest decomposition scale (default: 3).
%	qmfRi	QMF regularity (number of vanishing moments, default: 6 and 2).
%	regpar	regularization parameter (typically, the value 0.1 gives excellent estimates).
%	thresh	threshold factor used in for hard thresholding at 1st stage in WDWF (default: 3).
%	sigma	the noise std (default: estimated from y).
%	psd	the PSD of the input (default: estimated from y).
% Output
%	x_ward	deconvolved image.

if (exist('y')~=1), 	error('Provide input image');	end;
if (exist('h')~=1), 	error('Provide input PSF');	end;
if (exist('jc')~=1), 	jc = 3;				end;
if (exist('qmfR1')~=1), qmfR1 = 6;			end;
if (exist('qmfR2')~=1), qmfR2 = 2;			end;
if (exist('regpar')~=1),regpar = 0.1;			end;
if (exist('thresh')~=1),thresh = 3;			end;

[nx ny] = size(y);
if nx~=ny
	disp('Matrix must be square');
	return;
end
n 	= nx;
J	= nextpow2(n);
qmf1 = daubcqf(qmfR1,'min'); 
qmf2 = daubcqf(qmfR2,'min'); 

if (exist('sigma')~=1),
 buf	= FWT2_PO(y,J-1,qmf1);
 hg	= buf(end/2+1:end,end/2+1:end);
 sigma	= MAD(hg(:));
 clear hg;
end

FTY = fft2(y);
G   = fft2(h);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Power spectral density estimation
% This might not be robust for certain images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (exist('psd')~=1),
  % Note: We add an offset to the PSD estimate to stabilize it at frequencies
  % where the convolution operator response goes to zero.
  Iterations = 15;
  delta = 8*10^-8;
  Pyy = abs(FTY).^2;
  Pxx = Pyy;
  Phh = abs(G).^2;
  for i = 1:Iterations
    M = (conj(G) .* Pxx .* FTY) ./ (Phh .* Pxx + n^2*(sigma^2));
    Pxx_y = (Pxx .* n^2*sigma^2) ./ (Phh.*Pxx + n^2*(sigma^2));
    Pxx = Pxx_y + abs(M).^2;
    Norm_Pxx(i) = norm(Pxx(:));
  end
  Pxx = (Pxx+delta*max(Pyy(:))) *norm(Pxx(:))/norm(Pxx(:) + delta*max(Pyy(:)));
  temp = Pyy(2:256,2:256);  
  Weight = sqrt(Pxx);
end
clear Pyy Pxx Phh

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wiener Estimate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Wiener_filter_Est = (conj(G) .* (Weight.^2)) ./ ((abs(G).^2 .* (Weight.^2)) + n^2*(sigma^2));
Wiener_Estimate  = real(ifft2(Wiener_filter_Est .* FTY));
clear Wiener_filter_Est

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Regularized inversion in WaRD. This corresponds to the FoRD step.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Reg_Inv_filter = (conj(G) .* (Weight.^2)) ./ ((abs(G).^2 .* (Weight.^2)) + regpar*n^2*(sigma^2));
x_ford = real(ifft2(Reg_Inv_filter .* FTY));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet domian wiener filtering 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Take the RDWT of the x_ford
[x_fordw, x_fords] = mrdwt_cycle2D(x_ford,qmf1,jc);

% Calculate the variances of the noise at different levels
temp_filt = real(ifft2(Reg_Inv_filter));
[temp_w,temp_s] = mrdwt_cycle2D(temp_filt,qmf1,jc);
for lev = 1:jc
  for sub = 1:3
    noise_varw(lev,sub) = norm(temp_w(:,:,lev,sub),'fro')*sigma;
  end
end
noise_vars = norm(temp_s(:),2)*sigma;
clear temp_w temp_s 

% Step 1 in WDWF: Get the hard thresholded estimate

for lev = 1:jc
  for sub = 1:3
    temp_w(:,:,lev,sub)=HardThresh(x_fordw(:,:,lev,sub),...
	thresh*noise_varw(lev,sub)); %thresh difft wavelet subbands
  end
end
temp_s = HardThresh(x_fords,thresh*noise_vars); % thresh scaling subband
clear x_fordw x_fords
x_ref = mirdwt_cycle2D(temp_w,temp_s,qmf1,jc);
clear temp_w temp_s

% Use the estimate to perform wavelet domain wiener filtering.

% Calculate the variances of the noise at different levels
[temp_w,temp_s] = mrdwt_cycle2D(temp_filt,qmf2,jc);
for lev = 1:jc
  for sub = 1:3
    noise_varw(lev,sub) = norm(temp_w(:,:,lev,sub),'fro')*sigma;
  end
end
noise_vars = norm(temp_s(:),2)*sigma;
clear temp_w temp_s temp_filt

[x_fordw, x_fords] = mrdwt_cycle2D(x_ford,qmf2,jc);
[x_refw, x_refs] = mrdwt_cycle2D(x_ref,qmf2,jc);

% Do the wavelet domain wiener filtering

for lev = 1:jc
  for sub = 1:3
    temp_w(:,:,lev,sub)=x_fordw(:,:,lev,sub) .*...
	((x_refw(:,:,lev,sub).^2)./...
	(x_refw(:,:,lev,sub).^2 + noise_varw(lev,sub)^2)); 
  end
end
temp_s =x_fords .* ((x_refs.^2)./(x_refs.^2 + noise_vars^2)); 
clear x_fordw x_fords x_refw x_refs

x_ward = mirdwt_cycle2D(temp_w,temp_s,qmf2,jc);
clear temp_w temp_s





