function xest = proximaliteration(y, psf, lambda, mu0, Wavelet_Filt, Wavelet_Supp, sc, Orthogonal, Iterations)
% Proximal iteration deconvolution.
% Degradation equation:
%               y = h*x + eps, 
%		y	Observed image.
%		x	Original image to be recovered whose PSD is Sxx(w).
%		h	PSF (blurring function IR): assumed to be centred at the image center (M/2,N/2) and normalized to a unit mass.
%		*	Circular convolution.
%		eps	Stationary gaussian zero-mean noise with PSD See(w) (i.e. here assumed white gaussian).
%		lambda  Regularization parameter.
%		mu0	Relaxation parameter (gradient-descent: >0 and <2 ).
%		Wavelet* Wavelet QMF properties.
%		sc	Coarsest decomposition scale.
%		Orthogonal	Type of wavelet transform (0: decimated, 1: undecimated).
%		Iterations	Number of iterations.
%		
%
% We here minimize the functional:
%	0.5 || y - H Phi alpha||_2^2 + lambda ||alpha||_1
% where Phi is the dictionary. We here implement the DWT or the UDWT. Strictly speaking, the algorithm is only valid for tight frames and
% bases with dual synthesis frame reconstruction operator (pseudo-inverse). You may also use cycle-spinning.
%
% This is solved by the proximal iteration:
%
%
%             alpha^(t+1) = ST_\lambda*mu^(t) [alpha^(t) + mu^(t) Phi^* H^* (y - H Phi alpha^(t))]
%
% ST_mu is the soft thresholding operator with threshold mu,
% such that 0 < inf_t mu^(t) <= sup_t mu^(t) < 2/||Phi H||^2 = 2/(B sum(h(:))^2) for h a low-pass filter, B is the frame Phi upper-bound.
% 

[N,J] = quadlength(y);
N2 = N*N;

if ~exist('psf')
	error('Blur PSF is required.');
end
if length(psf) ~= N
	error('Blur PSF must have the same size as observed image.');
end
if ~exist('mu0')
	mu0 = 1.99;
end
if ~exist('Wavelet_Filt')
	Wavelet_Filt = 'Haar';
end
if ~exist('Wavelet_Supp')
	Wavelet_Supp = 1;
end
if ~exist('sc')
	sc = J-3;
end
if ~exist('Orthogonal')
	Orthogonal = 1;
end
if ~exist('Iterations')
	Iterations = 20;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define the wavelet systems for denoising step. 
% Redundant wavelet transforms are used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
qmf = MakeONFilter('Daubechies',4);
[temp_s,temp_w] = mrdwt(y,qmf,1);
clear temp_s

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Which transform to be used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Orthogonal, 
	Transform = 'DWT';
	tmp = ones(N,N);
	tmp(1:2^sc,1:2^sc) = 0;
	Details = find(tmp(:));
	clear tmp
else	       
	Transform = 'UDWT'; % This is not a tigth frame, but still works.
	Details = N2+1:(3*sc+1)*N2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Noise variance estimation using median estimator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma = MAD(temp_w(:,2*N+1:3*N));
clear temp_s temp_w

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start the proximal iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
qmf = MakeONFilter(Wavelet_Filt,Wavelet_Supp);
FTY = fft2(y);
H   = fft2(psf);
[psfll,psfwc] = mrdwt(psf, qmf, sc);
mu  = relaxstep(mu0,H,sc,Orthogonal);
%mu = 1.99/max(abs(H(:)).^2);
%xest = real(ifft2(FTY .* conj(H) * 1E3./ (abs(H).^2 * 1E3 + N^2*sigma^2))); % rather arbitrary.
xest = wienerdeconviter(y,psf,N2*sigma^2,20); % Initialize with the iterative Wiener filter.
%xest = landweberdeconv(y,psf,1,20);
C = eval([Transform, 'Analysis(xest,qmf,sc)']);
for i = 1:Iterations
    Xest = fft2(xest);
    residual = real(ifft2(conj(H) .* (FTY - H.*Xest)));
    C = C + mu.*eval([Transform, 'Analysis(residual,qmf,sc)']); % This is the correct iteration for a frame.
    C(Details) = SoftThresh(C(Details), lambda*mu(Details));          % Typically lambda = 0.25*sigma
    %C(Details) = max(C(Details).^2 - lambda,0)./C(Details); % Typically lambda = 3*Sigma^2
    xest = eval([Transform, 'Synthesis(C,N,qmf,sc)']);
    %energy(i) = 0.5*norm((FTY - H.*fft2(xest))/N,'fro')^2+lambda*norm(C,1);
    %plot(energy);drawnow;
    %imagesc(xest);axis image;rmaxis;colorbar;drawnow
end

% Analysis operator.
function c = DWTAnalysis(img, qmf, scale)

[n,J] = quadlength(img);
c = FWT2_PO(img, scale, qmf);
c = c(:);

% Analysis operator.
function c = UDWTAnalysis(img, qmf, scale)

[n,J] = quadlength(img);
[ll,wc] = mrdwt(img, qmf, scale);
c = [ll(:);wc(:)];

    
% Synthesis operator.
function img = DWTSynthesis(c, n, qmf, scale)

c = reshape(c, n, n);
img = IWT2_PO(c, scale, qmf);

% Synthesis operator.
function img = UDWTSynthesis(c, n, qmf, scale)

J = nextpow2(n);
ll = reshape(c(1:n*n),n,n);
wc = reshape(c(n*n+1:end),n,3*scale*n);
img = mirdwt(ll, wc, qmf, scale);

% Adaptive relaxation step (assuming Shannon wavelets).
function mu=relaxstep(mu0,H,sc,Orthogonal)

n = length(H);
if ~Orthogonal
 mu = max(max(abs(H(1:2^(sc-1),1:2^(sc-1)))))^2;
 for j = 1:sc
  tmp = H(n*2^(-j-1)+1 : n*2^(-j), 1:n*2^(-j-1));
  mu = [mu max(abs(tmp(:)))^2];
 
  tmp = H(1:n*2^(-j-1), n*2^(-j-1)+1 : n*2^(-j));
  mu = [mu max(abs(tmp(:)))^2];
 
  tmp = H(n*2^(-j-1)+1 : n*2^(-j), n*2^(-j-1)+1 : n*2^(-j));
  mu = [mu max(abs(tmp(:)))^2];
 end
 mu = repmat(mu0./mu,n*n,1);
else
 mu = zeros(size(H));
 mu(1:2^sc,1:2^sc) = max(max(abs(H(1:2^(sc-1),1:2^(sc-1)))))^2;
 for j = 1:sc
  tmp = H(n*2^(-j-1)+1 : n*2^(-j), 1:n*2^(-j-1));
  mu(n*2^(-j)+1 : n*2^(-j+1), 1:n*2^(-j)) = max(abs(tmp(:)))^2;
 
  tmp = H(1:n*2^(-j-1), n*2^(-j-1)+1 : n*2^(-j));
  mu(1:n*2^(-j), n*2^(-j)+1 : n*2^(-j+1)) = max(abs(tmp(:)))^2;
 
  tmp = H(n*2^(-j-1)+1 : n*2^(-j), n*2^(-j-1)+1 : n*2^(-j));
  mu(n*2^(-j)+1 : n*2^(-j+1), n*2^(-j)+1 : n*2^(-j+1)) = max(abs(tmp(:)))^2;
 end
 mu = mu0./mu;
end
mu = mu(:);
