function [imdeconv,rpgcv,regpar,gcv] = regdeconv(y,psf,usegcv,lambda)
% Tikhonov regularization operator for image deconvolution.
% Degradation equation:
%               y = h*x + eps, 
%		y	Observed image.
%		x	Original image to be recovered whose PSD is Sxx(w).
%		h	PSF (blurring function IR): assumed to be centred at the image center 
%			(M/2,N/2) and normalized to a unit mass.
%		*	Circular convolution.
%		eps	Stationary gaussian zero-mean noise with PSD See(w) (i.e. here assumed white gaussian).
%
% We here implement the 2nd order Tikhonov regularizer expressed in the spectral domain:
%
%			                H*(w)
%             Xest(w) = Y(w). ---------------------
%                             |H(w)| + lambda Sll
%
%  where Sll is the PSD of discrete Laplacian operator, and lambda is a positive regularization weight.
%
%
% usegcv allows the use of GCV-based choice of the regularization parameter.
% Chooses a regularization parameter rpar chosen by generalized
% cross-validation function
%    
%                 RSS
%            G =  ---
%                 T^2
%
%    where T is an effective number of degrees of
%    freedom.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
[N,N] = size(y);

if ~exist('usegcv'), usegcv = 0; end

if ~exist('psf')
      error('Blur PSF is required.');
end

if length(psf) ~= N
      error('Blur PSF must have the same size as observed image.');
end

% Using FFT-based approach.
FTY = fft2(y);
G = fft2(psf);
Phh = abs(G).^2;
Regularization = 'Tikhonov';  % Or 'Ridge' for L2 norm penalization.
if strcmp(Regularization,'Tikhonov')
      Q = zeros(size(y));
      Q(1:3,1:3) = [0 1 0;1 -4 1;0 1 0];
elseif strcmp(Regularization,'Ridge')
      Q = 1;
end
FTQ = fft2(Q);
Pqq = abs(FTQ).^2;
n2=N^2;
htol	 = ((n2^2 + n2 + 1)*eps)^(1/2);
% Heuristic upper bound on regularization parameter
 hmax	  = max(sqrt(Phh(:)));   
% Heuristic lower bound
 hmin	  = min(sqrt(Phh(:))) * htol;

%regpar=linspace(hmin,5*hmax/1000,100);
if usegcv
      regpar=logspace(log10(hmin),log10(hmax),100);
else
      regpar=lambda;
end
%regpar=10.^regpar;
gcvmin=1.d+30;

for ir=1:length(regpar)
  lambda=regpar(ir);

% Solution of g
RegEst = FTY .* conj(G) ./ (Phh + lambda * Pqq);

% I - Hat matrix diagonal form.
IAhat = lambda * Pqq(:) ./ (Phh(:) + lambda * Pqq(:));

% RSS
rss(ir) 	    = sum(abs(FTY(:) .* IAhat).^2);

% DOF and T2
T2(ir)  	    = sum(IAhat)^2;

% GCV
gcv(ir)=rss(ir)/T2(ir);
 if gcv(ir) <= gcvmin
	gcvmin=gcv(ir);
	rpgcv=lambda;
 %else
%	break
 end
end

imdeconv = FTY .* conj(G) ./ (Phh + rpgcv * Pqq);
imdeconv = real(ifft2(imdeconv));
  
