function imdeconv = wienerdeconviter(y,psf,Snn,iterations)
% Iterative Wiener Filter as proposed in Hillery and Chin 1991.
% Degradation equation:
%               y = h*x + eps, 
%		y	Observed image.
%		x	Original image to be recovered whose PSD is Sxx(w).
%		h	PSF (blurring function IR): assumed to be centred at the image center (M/2,N/2) and normalized to a unit mass.
%		*	Circular convolution.
%		eps	Stationary gaussian zero-mean noise with PSD See(w) (i.e. ACF is TFD^-1[DSP]).
%
% We here implement an iterative Wiener filter [] expressed in the spectral domain:
%
%			                      Sxx(w)
%             Xest(w) = Y(w). H*(w).------------------------
%                                    |H(w)| Sxx(w) + Snn(w)
%			                      RSB(w)
%                     = Y(w). H*(w).------------------------
%                                    |H(w)| RSB(w) + 1
%
%	Snn is the nosie PSD. For a white noise with variance V, the PSD is simply a flat image.
%       e.g. Snn = N^2*(sigma^2) for an AWGN.
  
[N,N] = size(y);

if ~exist('psf')
      error('Blur PSF is required.');
end

if length(psf) ~= N
      error('Blur PSF must have the same size as observed image.');
end


% Start the iteration as proposed in Hillery and Chin 1991 [HC91].
delta = 8*10^-8;
FTY = fft2(y);
Syy = abs(FTY).^2;
Sxx = Syy;
G = fft2(psf);
Shh = abs(G).^2;
for i = 1:iterations
  Wiest = (FTY .* conj(G) .* Sxx) ./ (Shh .* Sxx + Snn);
  % Update the PSD Sxx of the estimate ([HC91] Eq.4.3).
  % Sxx_y = (Shh * Sxx^2) * (Syy - Shh * Sxx - See) / ((Shh*Sxx + See)^2)
  % Sxx = Sxx + Sxx_y
  Sxx_y = (Shh .* Sxx.^2) .* (Syy - Shh .* Sxx - Snn) ./ ((Shh.*Sxx + Snn).^2);
  Sxx = Sxx + Sxx_y;
end

% Note: add an offset to the PSD estimate to stabilize it at frequencies
% where the convolution operator response goes to zero.
Sxx = (Sxx+delta*max(Syy(:))) *norm(Sxx(:))/norm(Sxx(:) + delta*max(Syy(:)));
Wiest = (FTY .* conj(G) .* Sxx) ./ (Shh .* Sxx + Snn);

imdeconv = real(ifft2(Wiest));
