function w = FTWT2_YMTI(x,L,deg)
% FTWT2_YMTI -- 2D TI forward Wavelet Transform (periodized Meyer Wavelet)
%  Usage
%    w = FTWT2_YMTI(x,L,deg)
%  Inputs
%    x    2-d image; n x n (n = 2^J)
%    L    Coarsest Level of V_0;  L << J
%    deg  degree of polynomial window 2 <= deg <=4
%  Outputs
%    w    2-d wavelet transform of x
%
%  Description
%    The Meyer wavelet is defined in the frequency domain.
%    The algorithm is then implemented in the Fourier domain.
%
%  See Also
%    ITWT2_YMTI,
%
	if L < 3,
		'L must be >= 3.  Enter new value of L or hit Ctrl-C to break.'
		L = input('L = ?');
	end
%
	n = length(x);
	J = log2(n);
	xhat = fft2(x);
%
%  Compute the wavelet coefficients.
%

	phij = scaling_YM(L,J,deg)';
	w = real(ifft2(conj(phij*phij').*xhat));
	for j=L:J-2
  	  phij = scaling_YM(j,J,deg)';
	  psij = wavelet_YM(j,J,deg)';
	  w(:,end+1:end+n) = real(ifft2(conj(psij*phij').*xhat)); % HL.
	  w(:,end+1:end+n) = real(ifft2(conj(phij*psij').*xhat)); % LH.
	  w(:,end+1:end+n) = real(ifft2(conj(psij*psij').*xhat)); % HH.
	end
	phij = scaling_YM(J-1,J,deg)';
	psij = fine_YM(J-1,J,deg)';
	w(:,end+1:end+n) = real(ifft2(conj(psij*phij').*xhat)); % HL.
	w(:,end+1:end+n) = real(ifft2(conj(phij*psij').*xhat)); % LH.
	w(:,end+1:end+n) = real(ifft2(conj(psij*psij').*xhat)); % HH.
	w = w*n; % To normalize the atoms to a unit l2 norm.
   
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function phi = phiHAT(x,deg)
% auxillary function for computing Meyer scaling function
% in the Fourier domain

winh = WindowMeyer((3*abs(x)-1),deg);

cos_part = cos(pi*winh/2);

phi = zeros(1,length(x));
phi = cos_part.*(abs(x)>1/3 & abs(x)<=2/3);
phi(abs(x)<=1/3) = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = psiHAT(x,deg)
% auxillary function for computing Meyer wavelet function
% in the Fourier domain

wing = WindowMeyer((3*abs(x)-1),deg);
winh = WindowMeyer((3*abs(x)/2-1),deg);

gpart = (exp(-i*pi*x)).*sin(pi*wing/2);
hpart = (exp(-i*pi*x)).*cos(pi*winh/2);

y1 = gpart.*(abs(x)>1/3 & abs(x)<=2/3);
y2 = hpart.*(abs(x)>2/3 & abs(x)<=4/3);

psij = y1+y2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%3
function phij = scaling_YM(j,j_max,deg)
% Generate phiHAT(j,0)--the fft of the scaling function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;
aux1=phiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

phij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(-j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = wavelet_YM(j,j_max,deg)
% generate psiHAT(j,0)--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;

omega_neg = [-10/(2*pi):1/(2^j):-1/2^j];
n_omega = 2*length(omega_pos)-1;

n = 2^j_max-n_omega;

aux1 = psiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

psij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(-j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = fine_YM(j,j_max,deg)
% generate psiHAT--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).
% at the finest resolution level

omega = linspace(-pi,pi,2^(j_max));
phij = scaling_YM(j,j_max,deg);
psij = exp(-i*linspace(-pi,pi,2^(j_max))/2^j).*conj(fftshift(phij));
