function x = ITWT2_YMTI(w,L,deg)
% ITWT2_YMTI -- 2D TI inverse Wavelet Transform (periodized Meyer Wavelet)
%  Usage
%    x = ITWT2_YMTI(w,L,deg)
%  Inputs
%    w    2-d tensor wavelet transform [n by 3*(J-L+1)*n+1 array, n dyadic]
%    L    Coarsest Level of V_0;  L << J
%    deg  degree of polynomial window 2 <= deg <=4
%  Outputs
%    x    2-d reconstructed image
%
%  Description
%    The Meyer wavelet is defined in the frequency domain.
%    The algorithm is then implemented in the Fourier domain.
%
%  See Also
%    FTWT2_YMTI,
%
	if L < 3,
		'L must be >= 3.  Enter new value of L or hit Ctrl-C to break.'
		L = input('L = ?');
	end
%
	n = size(w,1);
	J = log2(n);
%
%  Reconstruct from the wavelet coefficients.
%

	phij = scaling_YM(L,J,deg)';
	xhat = (phij*phij').*fft2(w(:,1:n));
	ind = n;
	for j=L:J-2
  	  phij = scaling_YM(j,J,deg)';
	  psij = wavelet_YM(j,J,deg)';
	  xhat = xhat + (psij*phij').*fft2(w(:,ind+1:ind+n)); % HL.
	  ind = ind+n;
	  xhat = xhat + (phij*psij').*fft2(w(:,ind+1:ind+n)); % LH.
	  ind = ind+n;
	  xhat = xhat + (psij*psij').*fft2(w(:,ind+1:ind+n)); % HH.
	  ind = ind+n;
	end
	phij = scaling_YM(J-1,J,deg)';
	psij = fine_YM(J-1,J,deg)';
	xhat = xhat + (psij*phij').*fft2(w(:,ind+1:ind+n)); % HL.
	ind = ind+n;
	xhat = xhat + (phij*psij').*fft2(w(:,ind+1:ind+n)); % LH.
	ind = ind+n;
	xhat = xhat + (psij*psij').*fft2(w(:,ind+1:ind+n)); % HH.
	
	x = real(ifft2(xhat))/n;
   
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function phi = phiHAT(x,deg)
% auxillary function for computing Meyer scaling function
% in the Fourier domain

winh = WindowMeyer((3*abs(x)-1),deg);

cos_part = cos(pi*winh/2);

phi = zeros(1,length(x));
phi = cos_part.*(abs(x)>1/3 & abs(x)<=2/3);
phi(abs(x)<=1/3) = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = psiHAT(x,deg)
% auxillary function for computing Meyer wavelet function
% in the Fourier domain

wing = WindowMeyer((3*abs(x)-1),deg);
winh = WindowMeyer((3*abs(x)/2-1),deg);

gpart = (exp(-i*pi*x)).*sin(pi*wing/2);
hpart = (exp(-i*pi*x)).*cos(pi*winh/2);

y1 = gpart.*(abs(x)>1/3 & abs(x)<=2/3);
y2 = hpart.*(abs(x)>2/3 & abs(x)<=4/3);

psij = y1+y2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%3
function phij = scaling_YM(j,j_max,deg)
% Generate phiHAT(j,0)--the fft of the scaling function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;
aux1=phiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

phij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = wavelet_YM(j,j_max,deg)
% generate psiHAT(j,0)--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;

omega_neg = [-10/(2*pi):1/(2^j):-1/2^j];
n_omega = 2*length(omega_pos)-1;

n = 2^j_max-n_omega;

aux1 = psiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

psij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = fine_YM(j,j_max,deg)
% generate psiHAT--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).
% at the finest resolution level

omega = linspace(-pi,pi,2^(j_max));
phij = scaling_YM(j,j_max,deg);
psij = exp(-i*linspace(-pi,pi,2^(j_max))/2^j).*conj(fftshift(phij));
